@@ -22,6 +22,7 @@ import (
 
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/common/model"
+	apiv1 "k8s.io/api/core/v1"
 	v1 "k8s.io/api/networking/v1"
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/util/workqueue"
@@ -31,23 +32,27 @@ import (
 
 // Ingress implements discovery of Kubernetes ingress.
 type Ingress struct {
-	logger   *slog.Logger
-	informer cache.SharedInformer
-	store    cache.Store
-	queue    *workqueue.Type
+	logger                *slog.Logger
+	informer              cache.SharedIndexInformer
+	store                 cache.Store
+	queue                 *workqueue.Type
+	namespaceInf          cache.SharedInformer
+	withNamespaceMetadata bool
 }
 
 // NewIngress returns a new ingress discovery.
-func NewIngress(l *slog.Logger, inf cache.SharedInformer, eventCount *prometheus.CounterVec) *Ingress {
+func NewIngress(l *slog.Logger, inf cache.SharedIndexInformer, namespace cache.SharedInformer, eventCount *prometheus.CounterVec) *Ingress {
 	ingressAddCount := eventCount.WithLabelValues(RoleIngress.String(), MetricLabelRoleAdd)
 	ingressUpdateCount := eventCount.WithLabelValues(RoleIngress.String(), MetricLabelRoleUpdate)
 	ingressDeleteCount := eventCount.WithLabelValues(RoleIngress.String(), MetricLabelRoleDelete)
 
 	s := &Ingress{
-		logger:   l,
-		informer: inf,
-		store:    inf.GetStore(),
-		queue:    workqueue.NewNamed(RoleIngress.String()),
+		logger:                l,
+		informer:              inf,
+		store:                 inf.GetStore(),
+		queue:                 workqueue.NewNamed(RoleIngress.String()),
+		namespaceInf:          namespace,
+		withNamespaceMetadata: namespace != nil,
 	}
 
 	_, err := s.informer.AddEventHandler(cache.ResourceEventHandlerFuncs{
@@ -67,6 +72,21 @@ func NewIngress(l *slog.Logger, inf cache.SharedInformer, eventCount *prometheus
 	if err != nil {
 		l.Error("Error adding ingresses event handler.", "err", err)
 	}
+
+	if s.withNamespaceMetadata {
+		_, err = s.namespaceInf.AddEventHandler(cache.ResourceEventHandlerFuncs{
+			UpdateFunc: func(_, o interface{}) {
+				namespace := o.(*apiv1.Namespace)
+				s.enqueueNamespace(namespace.Name)
+			},
+			// Creation and deletion will trigger events for the change handlers of the resources within the namespace.
+			// No need to have additional handlers for them here.
+		})
+		if err != nil {
+			l.Error("Error adding namespaces event handler.", "err", err)
+		}
+	}
+
 	return s
 }
 
@@ -79,11 +99,28 @@ func (i *Ingress) enqueue(obj interface{}) {
 	i.queue.Add(key)
 }
 
+func (i *Ingress) enqueueNamespace(namespace string) {
+	ingresses, err := i.informer.GetIndexer().ByIndex(cache.NamespaceIndex, namespace)
+	if err != nil {
+		i.logger.Error("Error getting ingresses in namespace", "namespace", namespace, "err", err)
+		return
+	}
+
+	for _, ingress := range ingresses {
+		i.enqueue(ingress)
+	}
+}
+
 // Run implements the Discoverer interface.
 func (i *Ingress) Run(ctx context.Context, ch chan<- []*targetgroup.Group) {
 	defer i.queue.ShutDown()
 
-	if !cache.WaitForCacheSync(ctx.Done(), i.informer.HasSynced) {
+	cacheSyncs := []cache.InformerSynced{i.informer.HasSynced}
+	if i.withNamespaceMetadata {
+		cacheSyncs = append(cacheSyncs, i.namespaceInf.HasSynced)
+	}
+
+	if !cache.WaitForCacheSync(ctx.Done(), cacheSyncs...) {
 		if !errors.Is(ctx.Err(), context.Canceled) {
 			i.logger.Error("ingress informer unable to sync cache")
 		}
@@ -200,6 +237,10 @@ func (i *Ingress) buildIngress(ingress v1.Ingress) *targetgroup.Group {
 	}
 	tg.Labels = ingressLabels(ingress)
 
+	if i.withNamespaceMetadata {
+		tg.Labels = addNamespaceLabels(tg.Labels, i.namespaceInf, i.logger, ingress.Namespace)
+	}
+
 	for _, rule := range ingress.Spec.Rules {
 		scheme := "http"
 		paths := pathsFromIngressPaths(rulePaths(rule))
