@@ -26,9 +26,11 @@ func TestMain(m *testing.M) {
 	goleak.VerifyTestMain(m)
 }
 
+// TestNewDiscoveryError can fail if the DNS resolver mistakenly resolves the domain below.
+// See https://github.com/prometheus/prometheus/issues/16191 for a precedent.
 func TestNewDiscoveryError(t *testing.T) {
 	_, err := NewDiscovery(
-		[]string{"unreachable.test"},
+		[]string{"unreachable.invalid"},
 		time.Second, []string{"/"},
 		nil,
 		func(_ []byte, _ string) (model.LabelSet, error) { return nil, nil })
