@@ -283,6 +283,24 @@ var Functions = map[string]*Function{
 		ArgTypes:   []ValueType{ValueTypeMatrix},
 		ReturnType: ValueTypeVector,
 	},
+	"ts_of_max_over_time": {
+		Name:         "ts_of_max_over_time",
+		ArgTypes:     []ValueType{ValueTypeMatrix},
+		ReturnType:   ValueTypeVector,
+		Experimental: true,
+	},
+	"ts_of_min_over_time": {
+		Name:         "ts_of_min_over_time",
+		ArgTypes:     []ValueType{ValueTypeMatrix},
+		ReturnType:   ValueTypeVector,
+		Experimental: true,
+	},
+	"ts_of_last_over_time": {
+		Name:         "ts_of_last_over_time",
+		ArgTypes:     []ValueType{ValueTypeMatrix},
+		ReturnType:   ValueTypeVector,
+		Experimental: true,
+	},
 	"minute": {
 		Name:       "minute",
 		ArgTypes:   []ValueType{ValueTypeVector},
