@@ -691,6 +691,42 @@ var tests = []struct {
 				},
 				seriesDesc: true,
 			},
+			{
+				input: `{} {{sum:1}}+{{sum:0}}x2 {{sum:1}}+{{sum:0}}x3`,
+				expected: []Item{
+					{LEFT_BRACE, 0, `{`},
+					{RIGHT_BRACE, 1, `}`},
+					{SPACE, 2, ` `},
+					{OPEN_HIST, 3, `{{`},
+					{SUM_DESC, 5, `sum`},
+					{COLON, 8, `:`},
+					{NUMBER, 9, `1`},
+					{CLOSE_HIST, 10, `}}`},
+					{ADD, 12, `+`},
+					{OPEN_HIST, 13, `{{`},
+					{SUM_DESC, 15, `sum`},
+					{COLON, 18, `:`},
+					{NUMBER, 19, `0`},
+					{CLOSE_HIST, 20, `}}`},
+					{TIMES, 22, `x`},
+					{NUMBER, 23, `2`},
+					{SPACE, 24, ` `},
+					{OPEN_HIST, 25, `{{`},
+					{SUM_DESC, 27, `sum`},
+					{COLON, 30, `:`},
+					{NUMBER, 31, `1`},
+					{CLOSE_HIST, 32, `}}`},
+					{ADD, 34, `+`},
+					{OPEN_HIST, 35, `{{`},
+					{SUM_DESC, 37, `sum`},
+					{COLON, 40, `:`},
+					{NUMBER, 41, `0`},
+					{CLOSE_HIST, 42, `}}`},
+					{TIMES, 44, `x`},
+					{NUMBER, 45, `3`},
+				},
+				seriesDesc: true,
+			},
 		},
 	},
 	{
@@ -915,6 +951,10 @@ var tests = []struct {
 				input: `test:name{on!~"bar"}[:4s]`,
 				fail:  true,
 			},
+			{
+				input: `test:name{on!~"bar"}[1s:1s:1s]`,
+				fail:  true,
+			},
 		},
 	},
 }
