@@ -61,7 +61,7 @@ func TestStoreHTTPErrorHandling(t *testing.T) {
 
 	for _, test := range tests {
 		server := httptest.NewServer(
-			http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 				http.Error(w, longErrMessage, test.code)
 			}),
 		)
@@ -93,7 +93,7 @@ func TestStoreHTTPErrorHandling(t *testing.T) {
 func TestClientRetryAfter(t *testing.T) {
 	setupServer := func(statusCode int) *httptest.Server {
 		return httptest.NewServer(
-			http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 				w.Header().Set("Retry-After", "5")
 				http.Error(w, longErrMessage, statusCode)
 			}),
@@ -180,7 +180,7 @@ func TestClientCustomHeaders(t *testing.T) {
 
 	var called bool
 	server := httptest.NewServer(
-		http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		http.HandlerFunc(func(_ http.ResponseWriter, r *http.Request) {
 			called = true
 			receivedHeaders := r.Header
 			for name, value := range headersToSend {
@@ -220,10 +220,12 @@ func TestReadClient(t *testing.T) {
 		name                  string
 		query                 *prompb.Query
 		httpHandler           http.HandlerFunc
+		timeout               time.Duration
 		expectedLabels        []map[string]string
 		expectedSamples       [][]model.SamplePair
 		expectedErrorContains string
 		sortSeries            bool
+		unwrap                bool
 	}{
 		{
 			name:        "sorted sampled response",
@@ -271,7 +273,7 @@ func TestReadClient(t *testing.T) {
 				StartTimestampMs: 4000,
 				EndTimestampMs:   12000,
 			},
-			httpHandler: http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			httpHandler: http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 				w.Header().Set("Content-Type", "application/x-streamed-protobuf; proto=prometheus.ChunkedReadResponse")
 
 				flusher, ok := w.(http.Flusher)
@@ -324,11 +326,25 @@ func TestReadClient(t *testing.T) {
 		},
 		{
 			name: "unsupported content type",
-			httpHandler: http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			httpHandler: http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
 				w.Header().Set("Content-Type", "foobar")
 			}),
 			expectedErrorContains: "unsupported content type",
 		},
+		{
+			name:                  "timeout",
+			httpHandler:           delayedResponseHTTPHandler(t, 15*time.Millisecond),
+			timeout:               5 * time.Millisecond,
+			expectedErrorContains: "context deadline exceeded: request timed out after 5ms",
+		},
+		{
+			name: "unwrap error",
+			httpHandler: http.HandlerFunc(func(w http.ResponseWriter, _ *http.Request) {
+				http.Error(w, "test error", http.StatusBadRequest)
+			}),
+			expectedErrorContains: "test error",
+			unwrap:                true,
+		},
 	}
 
 	for _, test := range tests {
@@ -339,9 +355,13 @@ func TestReadClient(t *testing.T) {
 			u, err := url.Parse(server.URL)
 			require.NoError(t, err)
 
+			if test.timeout == 0 {
+				test.timeout = 5 * time.Second
+			}
+
 			conf := &ClientConfig{
 				URL:              &config_util.URL{URL: u},
-				Timeout:          model.Duration(5 * time.Second),
+				Timeout:          model.Duration(test.timeout),
 				ChunkedReadLimit: config.DefaultChunkedReadLimit,
 			}
 			c, err := NewReadClient("test", conf)
@@ -355,6 +375,10 @@ func TestReadClient(t *testing.T) {
 			ss, err := c.Read(context.Background(), query, test.sortSeries)
 			if test.expectedErrorContains != "" {
 				require.ErrorContains(t, err, test.expectedErrorContains)
+				if test.unwrap {
+					err = errors.Unwrap(err)
+					require.EqualError(t, err, test.expectedErrorContains)
+				}
 				return
 			}
 
@@ -399,7 +423,7 @@ func TestReadClient(t *testing.T) {
 }
 
 func sampledResponseHTTPHandler(t *testing.T) http.HandlerFunc {
-	return func(w http.ResponseWriter, r *http.Request) {
+	return func(w http.ResponseWriter, _ *http.Request) {
 		w.Header().Set("Content-Type", "application/x-protobuf")
 
 		resp := prompb.ReadResponse{
@@ -437,3 +461,16 @@ func sampledResponseHTTPHandler(t *testing.T) http.HandlerFunc {
 		require.NoError(t, err)
 	}
 }
+
+func delayedResponseHTTPHandler(t *testing.T, delay time.Duration) http.HandlerFunc {
+	return func(w http.ResponseWriter, _ *http.Request) {
+		time.Sleep(delay)
+
+		w.Header().Set("Content-Type", "application/x-protobuf")
+		b, err := proto.Marshal(&prompb.ReadResponse{})
+		require.NoError(t, err)
+
+		_, err = w.Write(snappy.Encode(nil, b))
+		require.NoError(t, err)
+	}
+}
