@@ -263,6 +263,17 @@ func NewTemplateExpander(
 
 				return floatToTime(v)
 			},
+			"toDuration": func(i interface{}) (*time.Duration, error) {
+				v, err := common_templates.ConvertToFloat(i)
+				if err != nil {
+					return nil, err
+				}
+				d := time.Duration(v * float64(time.Second))
+				return &d, nil
+			},
+			"now": func() float64 {
+				return float64(timestamp) / 1000.0
+			},
 			"pathPrefix": func() string {
 				return externalURL.Path
 			},
@@ -270,7 +281,7 @@ func NewTemplateExpander(
 				return externalURL.String()
 			},
 			"parseDuration": func(d string) (float64, error) {
-				v, err := model.ParseDuration(d)
+				v, err := model.ParseDurationAllowNegative(d)
 				if err != nil {
 					return 0, err
 				}
