@@ -61,6 +61,9 @@ import {
   MadOverTime,
   MaxOverTime,
   MinOverTime,
+  TsOfMaxOverTime,
+  TsOfMinOverTime,
+  TsOfLastOverTime,
   Minute,
   Month,
   Pi,
@@ -403,6 +406,24 @@ const promqlFunctions: { [key: number]: PromQLFunction } = {
     variadic: 0,
     returnType: ValueType.vector,
   },
+  [TsOfMaxOverTime]: {
+    name: 'ts_of_max_over_time',
+    argTypes: [ValueType.matrix],
+    variadic: 0,
+    returnType: ValueType.vector,
+  },
+  [TsOfMinOverTime]: {
+    name: 'ts_of_min_over_time',
+    argTypes: [ValueType.matrix],
+    variadic: 0,
+    returnType: ValueType.vector,
+  },
+  [TsOfLastOverTime]: {
+    name: 'ts_of_last_over_time',
+    argTypes: [ValueType.matrix],
+    variadic: 0,
+    returnType: ValueType.vector,
+  },
   [Minute]: {
     name: 'minute',
     argTypes: [ValueType.vector],
